/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <gpxe/hp_ocsd.h>
#include <gpxe/init.h>
#include <gpxe/instance.h>
#include <realmode.h>
#include <arch/i386/include/registers.h>
#include <gpxe/netdevice.h>
#include <gpxe/pci.h>
#include <gpxe/device.h>

struct hp_info {
	uint16_t hp_boot_type;
} __attribute__((packed));


__asmcall void hp_features(struct i386_all_regs *ix86)
{
	uint32_t pci_busdevfn_from_prefix = instance_pci_busdevfn();
	struct net_device *netdev;
	struct pci_device *pci;
	struct hp_info hp_info;
	int rc = 0;

	instance_set_pci_busdevfn(ix86);

	/* The driver will take care of OCSD */
	initialise();
	startup();

	netdev = find_netdev_by_location(BUS_TYPE_PCI,
					 pci_busdevfn_from_prefix);
	if (!netdev) {
		DBG("No netdevice found for PCI address %x\n",
		    pci_busdevfn_from_prefix);
		goto out;
	}

	pci = container_of(netdev->dev, struct pci_device, dev);
	if (!is_hp(pci)) {
		DBG("This is not an HP device\n");
		goto out;
	}

	/* Setup hp_info for passing information back to the prefix */
	copy_from_real(&hp_info, ix86->segs.ds, ix86->regs.edi,
		       sizeof(hp_info));

	if (netdev->op->sf_get_boot_type)
		rc = netdev->op->sf_get_boot_type(netdev);

	if (rc < 0)
		goto out;
	else
		hp_info.hp_boot_type = rc;

	copy_to_real(ix86->segs.ds, ix86->regs.edi, &hp_info, sizeof(hp_info));

out:
	shutdown_exit();
}
