/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
#ifndef _SFCONFIG_BRIEF_WIDGETS_H
#define _SFCONFIG_BRIEF_WIDGETS_H

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <gpxe/settings.h>
#include <gpxe/sf_widgets.h>
#include <config/general.h>
#include <gpxe/sf_main.h>

/** @file
 *
 * Routines to work with brief widgets descriptions
 */

/* Configuration window */
#define SFC_INIT_ROW		2	/* Row to start locating widgets */
#define SFC_INIT_COL		4	/* Left margin of widgets */
#define SFC_EDITBOX_WIDTH	20	/* Default editboxes width */

/* Configuration window parameters */
#define SF_WIN_ROW	0
#define SF_WIN_COLUMN	0
#define SF_WIN_HEIGHT	24
#define SF_WIN_WIDTH	80

/* Window parameters */
#define WINDOW_LEVEL0_TOP 0
#define WINDOW_LEVEL0_LEFT 0
#define WINDOW_LEVEL0_WIDTH SF_WIN_WIDTH
#define WINDOW_LEVEL0_HEIGHT SF_WIN_HEIGHT
#define WINDOW_LEVEL1_TOP 3
#define WINDOW_LEVEL1_LEFT 15
#define WINDOW_LEVEL1_WIDTH 50
#define WINDOW_LEVEL1_HEIGHT 18
#define WINDOW_LEVEL2_TOP 4
#define WINDOW_LEVEL2_LEFT 17
#define WINDOW_LEVEL2_WIDTH 50
#define WINDOW_LEVEL2_HEIGHT 18

/* 
 * Brief TUI widget description.
 *
 * array of such structures is 'compiled' to a list of wealthy sf_widget 
 * structures by prepare_conf_widgets() function
 */
struct sf_widget_brief {
	sf_widget_type  type;
	const char	*const text;
	const char	*const help;
	unsigned char	left, top, width, height;
	enum sf_settings settings;
	struct setting	*setting;
	int		aux_data2;

	int		( * action )( struct sf_widget *this );
};

extern int br_col, br_row;

#ifdef SFCONFIG_NOHELP
	#define HELP( _text ) NULL
#else
	#define HELP( _text ) ( _text )
#endif

#define SF_BR_WINDOW( _level, _text )					\
	{								\
		.type = SFW_WINDOW,					\
		.text = ( _text ),					\
		.left = ( WINDOW_LEVEL ## _level ## _LEFT ),		\
		.top = ( WINDOW_LEVEL ## _level ## _TOP ),		\
		.width = ( WINDOW_LEVEL ## _level ## _WIDTH ),		\
		.height = ( WINDOW_LEVEL ## _level ## _HEIGHT ),	\
	}

#define SF_BR_TEXT( _text )			\
	{					\
		.type = SFW_TEXT,		\
		.text = ( _text ),		\
	}
#define SF_BR_NEWLINE( _len )			\
	{					\
		.type = SFW_NEWLINE,		\
		.height = ( _len ),		\
	}
#define SF_BR_BUTTON( _text, _left, _height, _action, _aux,	\
		      _help )					\
	{							\
		.type = SFW_BUTTON,				\
		.text = ( _text ),				\
		.left = ( _left ),				\
		.height = ( _height ),				\
		.help = HELP ( _help ),   			\
		.action = ( _action ),				\
		.aux_data2 = ( _aux ),				\
	}
#define SF_BR_COMBO( _text, _len, _settings, _setting, _action, _help )	\
	{							\
		.type = SFW_COMBO,				\
		.text = ( _text ),				\
		.height = ( _len ),				\
		.settings = ( _settings ),			\
		.setting = ( _setting ),			\
		.action = ( _action ),				\
		.help =  HELP ( _help ),			\
	}
#define SF_BR_CHECKBOX( _text, _settings, _setting, _action, _help )	\
	{							\
		.type = SFW_CHECKBOX,				\
		.text = ( _text ),				\
		.settings = ( _settings ),			\
		.setting = ( _setting ),			\
		.action = ( _action ),				\
		.help = HELP ( _help ),				\
	}
#define SF_BR_EDITBOX( _text, _len, _settings, _setting, _action, _help )	\
	{							\
		.type = SFW_EDITBOX,				\
		.text = ( _text ),				\
		.width = ( _len ),				\
		.settings = ( _settings ),			\
		.setting = ( _setting ),			\
		.action = ( _action ),				\
		.help = HELP ( _help ),				\
	}
#define SF_BR_PASSWORD( _text, _len, _settings, _setting, _action, _help )	\
	{							\
		.type = SFW_EDITBOX,				\
		.text = ( _text ),				\
		.top = '*',					\
		.width = ( _len ),				\
		.settings = ( _settings ),			\
		.setting = ( _setting ),			\
		.action = ( _action ),				\
		.help = HELP ( _help ),				\
	}
#define SF_BR_INFO( _text )					\
	{							\
		.type = SFW_INFO,				\
		.text = ( _text ),				\
	}
#define SF_BR_LIST( _rows, _left )				\
	{							\
		.type = SFW_LIST,				\
		.height = ( _rows ),				\
		.left = ( _left ),				\
	}
#define SF_BR_ROW( _text, _settings, _setting, _help )			\
	{							\
		.type = SFW_ROW,				\
		.text = ( _text ),				\
		.settings = ( _settings ),			\
		.setting = ( _setting ),			\
		.help = HELP ( _help ),				\
	}

/* Caption - A text item without an implicit newline */
#define SFW_CAPTION (SFW_NITEMS)
#define SF_BR_CAPTION( _text, _left, _height )			\
	{							\
		.type = SFW_CAPTION,				\
		.text = ( _text ),				\
		.left = ( _left ),				\
		.height = ( _height ),				\
	}

/* Prepare a widget list by an array of brief descriptions. */
extern sf_widget *
prepare_conf_widgets (  sf_widget *window, int margin, struct settings **settings,
			const struct sf_widget_brief *layout, int layout_size,
			const sf_option *option_list, void *aux_data );

#endif /* _SFCONFIG_BRIEF_WIDGETS_H */
