/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
#ifndef _GPXE_DYNCFG_H
#define _GPXE_DYNCFG_H

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <gpxe/settings.h>
#include <gpxe/nvs.h>
#include <stdint.h>
#include "tlv_layout.h"

#define MAX_VPD_V0	32

/* These are the NIC agnostic port modes as shown in the Ctrl-B menu */
enum sfc_port_mode {
	SFC_PORT_MODE_DEFAULT = TLV_PORT_MODE_DEFAULT,
	SFC_PORT_MODE_40G = TLV_PORT_MODE_40G,
	SFC_PORT_MODE_10G_10G = TLV_PORT_MODE_10G_10G,
	SFC_PORT_MODE_40G_40G = TLV_PORT_MODE_40G_40G,
	SFC_PORT_MODE_10G_10G_10G_10G = TLV_PORT_MODE_10G_10G_10G_10G,
	/* Avoid reusing NIC specific port mode numbers */
	SFC_PORT_MODE_4x10G = 20,
	SFC_PORT_MODE_100G,
	SFC_PORT_MODE_25G_25G,
	SFC_PORT_MODE_50G_50G_Q,
	SFC_PORT_MODE_4x25G,
	SFC_PORT_MODE_25G_25G_25G_25G,
	SFC_PORT_MODE_50G_50G,
	SFC_PORT_MODE_INVALID
};

/* All port modes supported in the Ctrl-B menu */
#define BIT(nr)				(1 << (nr))
#define SFC_SUPPORTED_PORT_MODES	(BIT(SFC_PORT_MODE_40G)		| \
					 BIT(SFC_PORT_MODE_10G_10G)	| \
					 BIT(SFC_PORT_MODE_40G_40G)	| \
					 BIT(SFC_PORT_MODE_10G_10G_10G_10G) | \
					 BIT(SFC_PORT_MODE_4x10G)	| \
					 BIT(SFC_PORT_MODE_100G)	| \
					 BIT(SFC_PORT_MODE_25G_25G)	| \
					 BIT(SFC_PORT_MODE_50G_50G_Q)	| \
					 BIT(SFC_PORT_MODE_4x25G)	| \
					 BIT(SFC_PORT_MODE_25G_25G_25G_25G) | \
					 BIT(SFC_PORT_MODE_50G_50G))

/* And with 0x0F20 so that Siena 0x0813 -> 0x0800 as before */
#define SFU_DEVICE_TYPE(pci_id) ((pci_id) & 0x0F20)

struct dyncfg {
	struct settings settings;
	/* Buffer storing a local copy of the dynamic config. */
	struct nvs_device nvs;

	/* Per function settings. */
	uint16_t msix_limit;
	uint16_t vf_msix_limit;
	uint8_t  vf_count;
	/* Global settings (only primary function should write these). */
	uint32_t firmware_variant;
	int16_t pf_count;
	uint32_t port_mode;
	enum sfc_port_mode generic_port_mode;
	uint32_t privilege_mask;
	uint8_t pfiov;
	uint8_t vlan_enabled;
	uint32_t vlan_tag;
	uint32_t vswitch_type;
	/* Indicates if one of the settings above has been modified. */
	unsigned settings_modified;
	uint8_t vpd_v0[MAX_VPD_V0];
};

extern int dyncfg_read ( struct settings *settings );
extern int dyncfg_changed ( struct settings *settings );
extern int dyncfg_flush ( struct settings *settings );

#endif
