/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
#ifndef	IGMP_H
#define	IGMP_H

#include "stdint.h"
#include <gpxe/in.h>

#define IGMP_QUERY	0x11
#define IGMPv1_REPORT	0x12
#define IGMPv2_REPORT	0x16
#define IGMP_LEAVE	0x17
#define GROUP_ALL_HOSTS 0xe0000001 /* 224.0.0.1 Host byte order */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#define MULTICAST_MASK    0xf0000000
#define MULTICAST_NETWORK 0xe0000000

enum {
	IGMP_SERVER,
	MAX_IGMP
};

struct igmp {
	uint8_t		type;
	uint8_t		response_time;
	uint16_t	chksum;
	struct in_addr	group;
} __attribute__ (( packed ));

struct igmp_ip_t { /* Format of an igmp ip packet */
	struct iphdr ip;
	uint8_t router_alert[4]; /* Router alert option */
	struct igmp igmp;
} __attribute__ (( packed ));

struct igmptable_t {
	struct in_addr group;
	unsigned long time;
} __attribute__ (( packed ));

extern void join_group ( int slot, unsigned long group );
extern void leave_group ( int slot );

#endif	/* IGMP_H */
